from __future__ import annotations

from typing import NotRequired, TypedDict

from omu.helper import map_optional
from omu.identifier import Identifier
from omu.interface import Keyable
from omu.model import Model

from .role import Role, RoleJson


class AuthorMetadata(TypedDict):
    url: NotRequired[str | None]
    screen_id: NotRequired[str | None]
    avatar_url: NotRequired[str | None]
    description: NotRequired[str | None]
    links: NotRequired[list[str] | None]


class AuthorJson(TypedDict):
    provider_id: str
    id: str
    metadata: AuthorMetadata
    name: NotRequired[str] | None
    avatar_url: NotRequired[str] | None
    roles: NotRequired[list[RoleJson]] | None


class Author(Keyable, Model[AuthorJson]):
    def __init__(
        self,
        *,
        provider_id: Identifier,
        id: Identifier,
        metadata: AuthorMetadata | None = None,
        name: str | None = None,
        avatar_url: str | None = None,
        roles: list[Role] | None = None,
    ) -> None:
        self.provider_id = provider_id
        self.id = id
        self.metadata = metadata or {}
        self.name = name
        self.avatar_url = avatar_url
        self.roles = roles or []

    def to_json(self) -> AuthorJson:
        return {
            "provider_id": self.provider_id.key(),
            "id": self.id.key(),
            "name": self.name,
            "avatar_url": self.avatar_url,
            "roles": [role.to_json() for role in self.roles],
            "metadata": self.metadata,
        }

    @classmethod
    def from_json(cls, json: AuthorJson) -> Author:
        return cls(
            provider_id=Identifier.from_key(json["provider_id"]),
            id=Identifier.from_key(json["id"]),
            name=json["name"],
            avatar_url=json.get("avatar_url"),
            roles=map_optional(
                json.get("roles"),
                lambda roles: list(map(Role.from_json, roles)),
                [],
            ),
            metadata=json.get("metadata", {}),
        )

    def key(self) -> str:
        return self.id.key()

    def __str__(self) -> str:
        return f"Author(id={self.id}, name={self.name})"



import re

import iwashi
from omu import Address, App, Omu
from omu.app import AppType

from .chat import (
    AUTHOR_TABLE,
    CHANNEL_TABLE,
    CREATE_CHANNEL_TREE_ENDPOINT,
    MESSAGE_TABLE,
    PLUGIN_ID,
    PROVIDER_TABLE,
    REACTION_SIGNAL,
    ROOM_TABLE,
    VOTE_TABLE,
)
from .model.channel import Channel
from .version import VERSION

app = App(
    id=PLUGIN_ID,
    version=VERSION,
    type=AppType.PLUGIN,
)
address = Address("127.0.0.1", 26423)
client = Omu(app, address=address)


messages = client.tables.get(MESSAGE_TABLE)
messages.set_config({"cache_size": 1000})
authors = client.tables.get(AUTHOR_TABLE)
authors.set_config({"cache_size": 500})
channels = client.tables.get(CHANNEL_TABLE)
providers = client.tables.get(PROVIDER_TABLE)
rooms = client.tables.get(ROOM_TABLE)
votes = client.tables.get(VOTE_TABLE)
reaction_signal = client.signals.get(REACTION_SIGNAL)


@client.endpoints.bind(endpoint_type=CREATE_CHANNEL_TREE_ENDPOINT)
async def create_channel_tree(url: str) -> list[Channel]:
    results = await iwashi.tree(url)
    if results is None:
        return []
    found_channels: dict[str, Channel] = {}
    services = await providers.fetch_all()
    for result in results.to_list():
        for provider in services.values():
            if re.search(provider.regex, result.url) is None:
                continue
            id = provider.id / result.id
            found_channels[id.key()] = Channel(
                provider_id=provider.id,
                id=id,
                name=result.name or result.id or result.service.name,
                description=result.description or "",
                icon_url=result.profile_picture or "",
                url=result.url,
                active=True,
            )
    return list(found_channels.values())
